/*************************************************** 
  This is an example for the Adafruit VS1053 Codec Breakout

  Designed specifically to work with the Adafruit VS1053 Codec Breakout 
  ----> https://www.adafruit.com/products/1381

  Adafruit invests time and resources providing this open source code, 
  please support Adafruit and open-source hardware by purchasing 
  products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries.  
  BSD license, all text above must be included in any redistribution

LCD connections:
 * LCD RS pin to digital pin 10
 * LCD Enable pin to digital pin 9
 * LCD D4 pin to digital pin 8
 * LCD D5 pin to digital pin 2
 * LCD D6 pin to digital pin 1
 * LCD D7 pin to digital pin 0
 * LCD R/W pin to ground
 * LCD VSS pin to ground
 * LCD VCC pin to 5V
 * 10K resistor:
 * ends to +5V and ground
 * wiper to LCD VO pin (pin 3)

 ****************************************************/

// include SPI, MP3 and SD libraries
#include <SPI.h>
#include <Adafruit_VS1053.h>
#include <SD.h>

// include LCD library code:
#include <LiquidCrystal.h>

// These are the pins used for the music maker shield
#define SHIELD_RESET  -1      // VS1053 reset pin (unused!)
#define SHIELD_CS     7      // VS1053 chip select pin (output)
#define SHIELD_DCS    6      // VS1053 Data/command select pin (output)

// These are common pins between breakout and shield
#define CARDCS 4     // Card chip select pin
// DREQ should be an Int pin, see http://arduino.cc/en/Reference/attachInterrupt
#define DREQ 3       // VS1053 Data request, ideally an Interrupt pin

Adafruit_VS1053_FilePlayer musicPlayer = 
  // create shield-example object!
  Adafruit_VS1053_FilePlayer(SHIELD_RESET, SHIELD_CS, SHIELD_DCS, DREQ, CARDCS);

// initialize the library by associating any needed LCD interface pin
// with the arduino pin number it is connected to
const int rs = 10, en = 9, d4 = 8, d5 = 2, d6 = 1, d7 = 0;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);
char line2[17];
char blankLine = "                ";

// The DEBUGGING flag activates printouts to the serial port. When debugging
// is active then the LCD stops working.
#define DEBUGGING true

/*
 * Define GPIO pins for buttons
 * https://learn.adafruit.com/adafruit-music-maker-shield-vs1053-mp3-wav-wave-ogg-vorbis-player?view=all#gpio-pins
 */
#define ON_BOARD_SWITCH 1 // Have a separate switch to do same thing as user's switch
#define USER_SWITCH 2 // Pin connected to jack for user's switch
#define VOLUME_UP 3
#define VOLUME_DOWN 4
#define DURATION_UP 5
#define DURATION_DOWN 6
#define NEXT_SONG 7

/*
 * How long does music play after each switch activation?
 */
// Max time (in msec) that music will play
#define MAX_DURATION 60
// Min time (in msec) that music will play
#define MIN_DURATION 1
// How much do we change duration with each button press?
#define DURATION_INCREMENT 1
// Length of time to delay after switch activation (in seconds)
int stimulusDuration = 30;

/*
 * How loud is the music?
 */
// Lower volume numbers are louder
#define MAX_VOLUME 0
// Higher volume numbers are quieter
#define MIN_VOLUME 100
// How much do we change volume with each button press?
#define VOLUME_INCREMENT 2
// Current volume
unsigned char volume = 40;

/*
 * Keep track of mp3 files stored on SD card
 */
// Max number of songs we will store
#define SONG_LIMIT 10
// We'll store filenames as arrays of characters.
// "Short" (8.3) filenames are used, followed by a null character.
char songFiles[SONG_LIMIT][13];
// Total number of songs stored
int songCount = 0;
// Which song is the current track?
int songIndex = 0;


void setup() {
  if (DEBUGGING) {
    Serial.begin(9600);
    Serial.println("Adafruit VS1053 Library Test");
  }

  // 
  // set up the LCD's number of columns and rows:
  //
  lcd.begin(16, 2);
  lcd.clear();

  // Print a message to the LCD.
  lcd.setCursor(0,0);
  lcd.print("Initializing...");

  //
  // Initialize the music player
  //
  if (! musicPlayer.begin()) { // initialise the music player
     if (DEBUGGING) Serial.println(F("Couldn't find VS1053, do you have the right pins defined?"));
     lcd.print("Couldn't find VS1053");
     while (1);
  }
  Serial.println(F("VS1053 found"));

  //
  // Set GPIO pins to input mode
  for (uint8_t i=1; i<=7; i++) 
    musicPlayer.GPIO_pinMode(i, INPUT);

  //
  // Initialize SD card
  //
  if (!SD.begin(CARDCS)) {
    if (DEBUGGING) Serial.println(F("SD failed, or not present"));
    lcd.print("SD failed, or not present");
    while (1);  // don't do anything more
  }
  if (DEBUGGING) Serial.println("SD OK!");
  
  //
  // get music files
  //
  getMusicFiles(SD.open("/"));

  if (DEBUGGING) {
    Serial.println("back from getMusicFiles");
    for(int x = 0; x < songCount; x++) {
      Serial.print(F("song "));
      Serial.print(x);
      Serial.print(F(": "));
      Serial.println(songFiles[x]);
    }
  }  
    
  //
  // Set volume for left, right channels. lower numbers == louder volume!
  //
  musicPlayer.setVolume(volume, volume);

  // This option uses a pin interrupt. No timers required! But DREQ
  // must be on an interrupt pin. For Uno/Duemilanove/Diecimilla
  // that's Digital #2 or #3
  // See http://arduino.cc/en/Reference/attachInterrupt for other pins
  // *** This method is preferred
  if (! musicPlayer.useInterrupt(VS1053_FILEPLAYER_PIN_INT)) {
    if (DEBUGGING) Serial.println(F("DREQ pin is not an interrupt pin"));
  }


  //
  // Queue up the first song
  //
  if (! musicPlayer.startPlayingFile(songFiles[songIndex])) {
    // If we have an error then stop the program
    if (DEBUGGING) {
      Serial.print("Could not open file ");
      Serial.println(songFiles[songIndex]);
    }
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Could not open ");
    lcd.print(songFiles[songIndex]);
    while (1);
  }
  musicPlayer.pausePlaying(true);

  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(songIndex);
  lcd.print(":");
  lcd.print(songFiles[songIndex]);

  if (DEBUGGING) {
    Serial.println("leaving setup()");
    for(int x = 0; x < songCount; x++) {
      Serial.print(F("song "));
      Serial.print(x);
      Serial.print(F(": "));
      Serial.println(songFiles[x]);
    }
  }  

}



void loop() {
  //
  // User activated switch
  //
  if (musicPlayer.GPIO_digitalRead(USER_SWITCH) || musicPlayer.GPIO_digitalRead(ON_BOARD_SWITCH)) {

    // User pressed the switch - wait until user releases switch
    while (musicPlayer.GPIO_digitalRead(USER_SWITCH) || musicPlayer.GPIO_digitalRead(ON_BOARD_SWITCH)) 
      delay(250);

    lcd.setCursor(0, 0);
    lcd.print(songFiles[songIndex]);
    
    if (musicPlayer.paused()) {
      // Player is paused - unpause music
      musicPlayer.pausePlaying(false);
      if (DEBUGGING) Serial.println(F("Unpaused song"));

    } else {
      // We are in-between songs - start a new song
      nextSong();
      musicPlayer.pausePlaying(false);
    }
    // clear second line of display
    // play song and display countdown    
    for (int totalPause = 0; totalPause < stimulusDuration; totalPause += 1) {
      // display countdown
      lcd.setCursor(0, 1);
      sprintf(line2, "Playing: %d.....", stimulusDuration - totalPause);
      lcd.print(line2);
      // play for a second
      delay(1000);
      // if the song is over, go to the next song
      if (musicPlayer.stopped()) {
        nextSong();
        musicPlayer.pausePlaying(false);
      }
    }
    // pause the song
    musicPlayer.pausePlaying(true);

  //
  // Increase song duration
  // 
  } else if (musicPlayer.GPIO_digitalRead(DURATION_UP)) {
    // Wait until user releases switch
    while (musicPlayer.GPIO_digitalRead(DURATION_UP)) delay(250);

    stimulusDuration += DURATION_INCREMENT;
    if (stimulusDuration > MAX_DURATION) stimulusDuration = MAX_DURATION;
    if (DEBUGGING) {
      Serial.print("Duration = ");
      Serial.println(stimulusDuration);
    }

  //
  // Decrease song duration
  // 
  } else if (musicPlayer.GPIO_digitalRead(DURATION_DOWN)) {
    // Wait until user releases switch
    while (musicPlayer.GPIO_digitalRead(DURATION_DOWN)) delay(250);

    if (DEBUGGING) {
      Serial.println("calling durationDown");
      for(int x = 0; x < songCount; x++) {
        Serial.print(F("song "));
        Serial.print(x);
        Serial.print(F(": "));
        Serial.println(songFiles[x]);
      }
    }  

    stimulusDuration -= DURATION_INCREMENT;
    if (stimulusDuration < MIN_DURATION) stimulusDuration = MIN_DURATION;
    if (DEBUGGING) {
      Serial.print("Duration = ");
      Serial.println(stimulusDuration);
    }

  //
  // Increase song volume - smaller numbers are louder
  // 
  } else if (musicPlayer.GPIO_digitalRead(VOLUME_UP)) {
    // Wait until user releases switch
    while (musicPlayer.GPIO_digitalRead(VOLUME_UP)) delay(250);

    volume -= VOLUME_INCREMENT;
    if (volume < MAX_VOLUME) volume = MAX_VOLUME;
    musicPlayer.setVolume(volume, volume);
    if (DEBUGGING) {
      Serial.print("Volume = ");
      Serial.println(volume);
    }

  //
  // Decrease song volume - bigger numbers are quieter
  // 
  } else if (musicPlayer.GPIO_digitalRead(VOLUME_DOWN)) {
    // Wait until user releases switch
    while (musicPlayer.GPIO_digitalRead(VOLUME_DOWN)) delay(250);

    volume += VOLUME_INCREMENT;
    if (volume > MIN_VOLUME) volume = MIN_VOLUME;
    musicPlayer.setVolume(volume, volume);
    if (DEBUGGING) {
      Serial.print("Volume = ");
      Serial.println(volume);
    }

  //
  // Go to next song
  //
  } else if (musicPlayer.GPIO_digitalRead(NEXT_SONG)) {
    // Wait until user releases switch
    while (musicPlayer.GPIO_digitalRead(NEXT_SONG)) delay(250);

    if (DEBUGGING) {
      Serial.println("calling nextSong");
      for(int x = 0; x < songCount; x++) {
        Serial.print(F("song "));
        Serial.print(x);
        Serial.print(F(": "));
        Serial.println(songFiles[x]);
      }
    }  

    musicPlayer.stopPlaying();
    nextSong();
    if (DEBUGGING) Serial.println("Moving to next track");
  }

  //
  // Display volume, duration and track
  //
  lcd.setCursor(0, 1);
  sprintf(line2, "Vol: %2d Dur: %2d    ", (100-volume), stimulusDuration);
  lcd.print(line2);
}


void nextSong() {
  if (DEBUGGING) {
    Serial.print(F("Entering nextSong, songIndex = "));  
    Serial.println(songIndex);
  }

  if (DEBUGGING) { // List all the files we found:
    for(int x = 0; x < songCount; x++) {
      Serial.print(F("song "));
      Serial.print(x);
      Serial.print(F(": "));
      Serial.println(songFiles[x]);
    }
  }  

  // move to next song in list of songs
  songIndex++;
  if (songIndex >= songCount) songIndex = 0; 
  if (DEBUGGING) {
    Serial.print(F("Incremented index, songIndex = "));  
    Serial.println(songIndex);
  }

  // If we have an error then stop the program
  while (! musicPlayer.startPlayingFile(songFiles[songIndex])) {
    if (DEBUGGING) {
      Serial.print("Could not open file ");
      Serial.println(songFiles[songIndex]);
    }
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Could not open song file");
    lcd.print(songFiles[songIndex]);
    if (DEBUGGING) {
      Serial.print(F("Could not open song file, songIndex = "));  
      Serial.print(songIndex);
      Serial.print(" filename = ");
      Serial.println(songFiles[songIndex]);     
    }
    // move to next song in list of songs
    songIndex++;
    if (songIndex >= songCount) songIndex = 0; 
  }
  musicPlayer.pausePlaying(true);

  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(songIndex);
  lcd.print(":");
  lcd.print(songFiles[songIndex]);
  if (DEBUGGING) {
    Serial.print(F("Leaving nextSong, songIndex = "));  
    Serial.println(songIndex);
  }  
}

void getMusicFiles(File dir) {
   // As long as we haven't filled up our list of songs...
   while(songCount < SONG_LIMIT) {
     // Get the next file
     File entry =  dir.openNextFile();
     // If there are no more files, then we're done
     if (! entry) break;
     // Print the name of the file
     if (DEBUGGING) {
       Serial.print(entry.name());
       Serial.print("\t\t");
       Serial.println(entry.size(), DEC);      
     }
     // Is this a playable file?
     if (isPlayable(entry.name())) {
       // Add filename to list of songs
       strcpy(songFiles[songCount],entry.name());
       songCount++;
     }
     // Close the file
     entry.close();
   }

  if (DEBUGGING) { // List all the files we found:
    for(int x = 0; x < songCount; x++) {
      Serial.print(F("song "));
      Serial.print(x);
      Serial.print(F(": "));
      Serial.println(songFiles[x]);
    }
  }  
}



boolean isPlayable(char track[]) {
  // Check to see if a filename has a "playable" extension.
  // This is to keep the VS1053 from locking up if it is sent
  // unplayable data.

  char *extension;

  // Ignore directory names
  if (track[0] == '.' || track[0] == '_') return false;

  // Find the first dot
  extension = strrchr(track,'.');
  extension++;

  if (DEBUGGING) {
    Serial.print("\tisPlayable: track = ");
    Serial.println(track);
  
    Serial.print("\tisPlayable: extension = ");
    Serial.println(extension);
  }

  // Is this a file that we can play?
  if (
    (strcasecmp(extension,"MP3") == 0) ||
    (strcasecmp(extension,"WAV") == 0) ||
    (strcasecmp(extension,"MID") == 0) ||
    (strcasecmp(extension,"MP4") == 0) ||
    (strcasecmp(extension,"WMA") == 0) ||
    (strcasecmp(extension,"FLA") == 0) ||
    (strcasecmp(extension,"OGG") == 0) ||
    (strcasecmp(extension,"AAC") == 0)
  )
    return true;
  else
    return false;
}
